 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include <stdint.h>
#include <vector>
#include <map>
#include <string>

#include "infiniband/ibutils/release_containers.h"

#include "uphy_common.h"
#include "uphy_version.h"
#include "uphy_dataset.h"



namespace UPHY
{
    class DB
    {
        private:
            typedef std::map<DataSetKey, const DataSet *> data_set_collections_t;

        private:
            data_set_collections_t m_datasets;

        private:
            DB() {};

        public:
            ~DB() {
                release_container_data(m_datasets);
            }

        public:
            inline static DB & instance() {
                static DB _instance;
                return _instance;
            }

        public:
            inline bool empty() const {
                return m_datasets.empty();
            }

        public:
            inline const DataSet * get(dataset_type_t type, const Version & version )
            {
                auto x = m_datasets.find(DataSetKey(type, version));
                return x != m_datasets.end() ? x->second : nullptr;
            }

        public:
            inline const DataSet * get_compatible(dataset_type_t type, const Version & version )
            {
                auto x = m_datasets.lower_bound(DataSetKey(type, version));
                return x == m_datasets.end() ? nullptr :
                        x->second->type() == type && x->second->version().major_version() == version.major_version() ? x->second : nullptr;
            }

        public:
            const DataSet * add(const DataSet * dataset)
            {
                if( dataset == nullptr )
                    return nullptr;

                auto x = m_datasets.insert(std::make_pair(DataSetKey(dataset->type(), dataset->version()), dataset));
                return x.second ? dataset : nullptr;
            }

    };
}
