 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include <string>
#include <sstream>

#include "uphy_common.h"

namespace UPHY
{
    class Version
    {
        public:
            typedef uint8_t ver_elem_t;

        private:
            const ver_elem_t m_major;
            const ver_elem_t m_minor;

        public:
            // '_version' suffix add by GNU issue (https://bugzilla.redhat.com/show_bug.cgi?id=130601)
            inline ver_elem_t major_version() const { return m_major; }
            inline ver_elem_t minor_version() const { return m_minor; }

        public:
            inline Version() = delete;
            inline Version(ver_elem_t major, ver_elem_t minor)
                : m_major(major), m_minor(minor)
            {}

        public:
            inline Version(const Version & version)
                : m_major(version.m_major), m_minor(version.m_minor)
            {}

        public:
            std::string to_string() const {
                std::stringstream sstr;
                sstr << +m_major << "." << +m_minor;
                return sstr.str();
            }

        public:
            inline bool operator == (const Version & rhs ) const {
                return m_major == rhs.m_major && m_minor == rhs.m_minor;
            }

            inline bool operator != (const Version & rhs ) const {
                return m_major != rhs.m_major || m_minor != rhs.m_minor;
            }

            inline bool operator < (const Version & rhs ) const {
                return m_major >= rhs.m_major ?
                        m_major == rhs.m_major && m_minor < rhs.m_minor : true;
            }

            inline bool operator > (const Version & rhs ) const {
                return m_major <= rhs.m_major ?
                        m_major == rhs.m_major && m_minor > rhs.m_minor : true;
            }
    };
}
